@@ -2,50 +2,40 @@
 base class for python applications with a graphical user interface
 ==================================================================
 
-The abstract base class :class:`MainAppBase` provided by this
-ae namespace portion allows the integration of any Python GUI
-framework into the ae namespace.
+The abstract base class :class:`MainAppBase` provided by this ae namespace portion allows the integration of any Python
+GUI framework into the ae namespace.
 
-On overview about the available GUI-framework-specific
-ae namespace portion implementations can be found
-in the documentation of the ae namespace portion
-:mod:`ae.lisz_app_data`.
+On overview about the available GUI-framework-specific ae namespace portion implementations can be found in the
+documentation of the ae namespace portion :mod:`ae.lisz_app_data`.
 
 
 extended console application environment
 ----------------------------------------
 
-The abstract base class :class:`MainAppBase` inherits directly from the ae namespace
-class :class:`ae console application environment class <ae.console.ConsoleApp>`.
-The so inherited helper methods are useful for to log, configure and
-control the run-time of your GUI app via command line arguments,
-:ref:`config-options` and :ref:`config-files`.
+The abstract base class :class:`MainAppBase` inherits directly from the ae namespace class
+:class:`ae console application environment class <ae.console.ConsoleApp>`. The so inherited helper methods are useful
+for to log, configure and control the run-time of your GUI app via command line arguments.
 
 .. hint::
-    Please see the documentation of the :mod:`ae.console` namespace
+    Please see the documentation of :ref:`config-options` and :ref:`config-files` in the :mod:`ae.console` namespace
     portion/module for more detailed information.
 
-:class:`MainAppBase` adds on top of the :class:`~ae.console.ConsoleApp`
-the concepts of :ref:`application status` and :ref:`application flow`,
-explained further down.
+:class:`MainAppBase` adds on top of the :class:`~ae.console.ConsoleApp` the concepts of :ref:`application status` and
+:ref:`application flow`, explained further down.
 
 
 application status
 ------------------
 
-Any application- and user-specific configurations like e.g. the last
-window position/size, the app theme/font/language or the last selected flow
-within your app, could be included in the application status.
+Any application- and user-specific configurations like e.g. the last window position/size, the app theme/font/language
+or the last selected flow within your app, could be included in the application status.
 
-This namespace portion introduces the section `aeAppState` in the app
-:ref:`config-files`, where any status values can be stored persistently
-for to be recovered on the next startup of your application.
+This namespace portion introduces the section `aeAppState` in the app :ref:`config-files`, where any status values can
+be stored persistently for to be recovered on the next startup of your application.
 
 .. hint::
-    The section name `aeAppState` is declared by the
-    :data:`APP_STATE_SECTION_NAME` constant. If you need
-    to access this config section directly then please
-    use this constant instead of the hardcoded section name.
+    The section name `aeAppState` is declared by the :data:`APP_STATE_SECTION_NAME` constant. If you need to access this
+    config section directly then please use this constant instead of the hardcoded section name.
 
 
 .. _app-state-variables:
@@ -63,44 +53,32 @@ This module is providing/pre-defining the following application state variables:
     * :attr:`~MainAppBase.sound_volume`
     * :attr:`~MainAppBase.win_rectangle`
 
-Which app state variables are finally used by your app project is (fully data-driven)
-depending on the app state :ref:`config-variables` detected in all the
-:ref:`config-files` that are found/available at run-time of your app.
-The names of all the available application state variables can be
-determined with the main app helper method :meth:`~MainAppBase.app_state_keys`.
+Which app state variables are finally used by your app project is (fully data-driven) depending on the app state
+:ref:`config-variables` detected in all the :ref:`config-files` that are found/available at run-time of your app.
+The names of all the available application state variables can be determined with the main app helper method
+:meth:`~MainAppBase.app_state_keys`.
 
-If your application is e.g. supporting a user-defined font size, using the
-provided/pre-defined app state variable :attr:`~MainAppBase.font_size`, then
-it has to call the method :meth:`change_app_state` with the
-:paramref:`~MainAppBase.change_app_state.state_name` set to `font_size`
-every time when the user has changed the font size of your app.
+If your application is e.g. supporting a user-defined font size, using the provided/pre-defined app state variable
+:attr:`~MainAppBase.font_size`, then it has to call the method :meth:`change_app_state` with the
+:paramref:`~MainAppBase.change_app_state.state_name` set to `font_size` every time when the user has changed the font
+size of your app.
 
 .. hint::
-    The two built-in app state variables are :attr:`~MainAppBase.flow_id` and
-    :attr:`~MainAppBase.flow_path` will be explained detailed in the next section.
+    The two built-in app state variables are :attr:`~MainAppBase.flow_id` and :attr:`~MainAppBase.flow_path` will be
+    explained detailed in the next section.
 
-The :meth:`~MainBaseApp.load_app_states` method is called on instantiation
-from the implemented main app class for to load the values of all
-app state variables from the :ref:`config-files`, and is then calling
-:meth:~MainAppBase.setup_app_states` for pass them into their corresponding
-instance attributes.
+The :meth:`~MainBaseApp.load_app_states` method is called on instantiation from the implemented main app class for to
+load the values of all app state variables from the :ref:`config-files`, and is then calling
+:meth:~MainAppBase.setup_app_states` for pass them into their corresponding instance attributes.
 
-Use the main app instance attribute for to read/get the actual value of
-a single app state variable. The actual values of
-all app state variables as a dict is determining the method
-:meth:`~MainBaseApp.retrieve_app_states`.
+Use the main app instance attribute for to read/get the actual value of a single app state variable. The actual values
+of all app state variables as a dict is determining the method :meth:`~MainBaseApp.retrieve_app_states`, and can be
+saved into the :ref:`config-files` for the next app run via the method :meth:`~MainBaseApp.save_app_states` - this could
+be done e.g. after the app state has changed or at least on quiting the application.
 
-Changed app state value that need to be propagated also to the framework
-app instance should never be set via the instance attribute, instead call
-the method :meth:`~MainBaseApp.change_app_state` (which ensures (1) the
-propagation to any duplicated (observable/bound) framework property and
-(2) the event notification to related the main app instance method).
-
-For to to save the app state to the :ref:`config-files` the
-implementing main app instance has to call the method
-:meth:`~MainBaseApp.save_app_states` - this could be
-done e.g. after the app state has changed or at least
-on quiting the application.
+Changed app state values have to be saved always by calling the method :meth:`~MainBaseApp.change_app_state` (which
+ensures (1) the propagation to any duplicated (observable/bound) framework property and (2) the event notification to
+related the main app instance method).
 
 
 .. _app-state-constants:
@@ -108,10 +86,8 @@ on quiting the application.
 app state constants
 ^^^^^^^^^^^^^^^^^^^
 
-This module is also providing some pre-defined constants
-that can be optionally used in your application in relation
-to the app states data store and for the app state config
-variables :attr:`~MainAppBase.font_size` and
+This module is also providing some pre-defined constants that can be optionally used in your application in relation
+to the app states data store and for the app state config variables :attr:`~MainAppBase.font_size` and
 :attr:`~MainAppBase.light_theme`:
 
     * :data:`APP_STATE_SECTION_NAME`
@@ -128,158 +104,114 @@ variables :attr:`~MainAppBase.font_size` and
 application flow
 ----------------
 
-For to control the current state and application/work flow
-(or context) of your application, and to persist it until
-the next app start, :class:`MainBaseApp` provides
-two :ref:`app-state-variables`.
-:attr:`~MainAppBase.flow_id` for to store the currently working flow
-and :attr:`~MainAppBase.flow_path` for to store the history of
-nested flows.
+For to control the current state and application/work flow (or context) of your application, and to persist it until
+the next app start, :class:`MainBaseApp` provides two :ref:`app-state-variables`: :attr:`~MainAppBase.flow_id` for to
+store the currently working flow and :attr:`~MainAppBase.flow_path` for to store the history of nested flows.
 
-A application flow is represented by an id string that defines
-three things: (1) the action for to enter into the flow, (2)
-the data or object that gets currently worked on and (3)
-an optional key string that is identifying/indexing
-a widget or data item of your application context/flow.
+A application flow is represented by an id string that defines three things: (1) the action for to enter into the flow,
+(2) the data or object that gets currently worked on and (3) an optional key string that is identifying/indexing a
+widget or data item of your application context/flow.
 
 .. note::
-    Never concatenate a flow id string manually, use the
-    :func:`id_of_flow` function instead.
+    Never concatenate a flow id string manually, use the :func:`id_of_flow` function instead.
 
-The flow id is initially an empty string. As soon as the user is
-starting a new work flow or the current selection your application
-should call the method :meth:`~MainBaseApp.change_flow` passing
-the flow id string into the
-:paramref:`~MainAppBase.change_flow.new_flow_id` argument
-for to change the app flow.
+The flow id is initially an empty string. As soon as the user is starting a new work flow or the current selection your
+application should call the method :meth:`~MainBaseApp.change_flow` passing the flow id string into the
+:paramref:`~MainAppBase.change_flow.new_flow_id` argument for to change the app flow.
 
-For more complex applications you can specify a path of nested flows.
-This flow path gets represented by the app state variable
-:attr:`~MainAppBase.flow_path`, which is a list of flow
-id strings.
+For more complex applications you can specify a path of nested flows. This flow path gets represented by the app state
+variable :attr:`~MainAppBase.flow_path`, which is a list of flow id strings.
 
-For to enter into a deeper/nested flow you simply call
-:meth:`~MainBaseApp.change_flow` with one of the actions
-defined in :data:`ACTIONS_EXTENDING_FLOW_PATH`.
+For to enter into a deeper/nested flow you simply call :meth:`~MainBaseApp.change_flow` with one of the actions defined
+in :data:`ACTIONS_EXTENDING_FLOW_PATH`.
 
-For to go back to a previous flow in the flow path
-call :meth:`~MainBaseApp.change_flow` passing one of the
-actions defined in :data:`ACTIONS_REDUCING_FLOW_PATH`.
+For to go back to a previous flow in the flow path call :meth:`~MainBaseApp.change_flow` passing one of the actions
+defined in :data:`ACTIONS_REDUCING_FLOW_PATH`.
 
 
 application events
 ------------------
 
-The following application events are fired exactly
-one time in the lifetime of an app instance in
-the following order:
+The following application events are fired exactly one time in the lifetime of an app instance in the following order:
 
-* `on_app_init`: fired **after** :class:`ConsoleApp`-parts
-  of the app instance got initialized (loaded config files
-  and app states)
-  and **before** the resources got loaded and the GUI
-  framework app class instance gets initialized.
-* `on_app_start`: fired **after** the call of the
-  method :meth:`~MainAppBase.run_app`
-  and **before** the command line options got parsed.
-* `on_app_run`: fired **after** the parsing of the
-  command line arguments and options and
-  **before** the framework event
+* `on_app_init`: fired **after** :class:`ConsoleApp`-parts of the app instance got initialized (detected config files)
+  and **before** the image and sound resources and app states get loaded and the GUI framework app class instance gets
+  initialized.
+* `on_app_start`: fired **after** the call of the method :meth:`~MainAppBase.run_app` and **before** the command line
+  options got parsed.
+* `on_app_run`: fired **after** the parsing of the command line arguments and options and **before** the framework event
   loop of the used GUI framework gets started.
-* `on_app_stop`: fired **after** the call of the
-  method :meth:`~MainAppBase.stop_app`
-  and **before** the framework win will
-  be closed.
-* `on_app_exit`: fired **after* framework win got closed and
-  just **before** the event loop of the GUI framework
-  will be stopped and the app shutdown.
-* `on_app_quit`: fired **after** the event loop of the GUI framework
-  got stopped and before the :meth:`AppBase.shutdown` method
-  will be called.
+* `on_app_stop`: fired **after** the call of the method :meth:`~MainAppBase.stop_app` and **before** the framework win
+  will be closed.
+* `on_app_exit`: fired **after* framework win got closed and just **before** the event loop of the GUI framework will be
+  stopped and the app shutdown.
+* `on_app_quit`: fired **after** the event loop of the GUI framework got stopped and before the :meth:`AppBase.shutdown`
+  method will be called.
 
 .. note::
-    The `on_app_stop` and `on_app_exit` events will only be
-    fired if the app is explicitly calling the
+    The `on_app_stop` and `on_app_exit` events will only be fired if the app is explicitly calling the
     :meth:`~MainAppBase.stop_app` method.
 
 
 application state change and save events
 ----------------------------------------
 
-Notification events get fired when one of the app state variables
-gets changed - like e.g. the :ref:`application flow` or if
-the user changes the font size.
+Notification events get fired when one of the app state variables gets changed - like e.g. the :ref:`application flow`
+or if the user changes the font size.
 
-The method name of the app state change notification event consists of
-the prefix ``on_`` followed by the variable name of the app state.
-So e.g. on a change of the `font_size` app state the notification
-event `on_font_size` will be fired/called (if exists as a method
-of the main app instance). These events don't provide any
-event arguments.
+The method name of the app state change notification event consists of the prefix ``on_`` followed by the variable name
+of the app state. So e.g. on a change of the `font_size` app state the notification event `on_font_size` will be
+fired/called (if exists as a method of the main app instance). These events don't provide any event arguments.
 
-Another event gets fired for each app state value just after the
-app states getting retrieved from the app class instance and
-before they get stored into the main config file.
+Another event gets fired for each app state value just after the app states getting retrieved from the app class
+instance and before they get stored into the main config file.
 
-This method name of this event includes the key/name of the
-app state using the format :meth:`on_app_state_<app_state_key>_save`,
-so e.g. for the app state `flow_id` the method name will result in
+This method name of this event includes the key/name of the app state using the format
+:meth:`on_app_state_<app_state_key>_save`, so e.g. for the app state `flow_id` the method name will result in
 :meth:`on_app_state_flow_id_save`.
 
-The event is taking one event argument with the value of the app state
-and if it returns a value that is not `None` then this value will
-be stored/saved.
+The event is taking one event argument with the value of the app state and if it returns a value that is not `None` then
+this value will be stored/saved.
 
 
 application flow change events
 ------------------------------
 
-The name of the event method that gets fired when the app want
-to change the flow (via the method :meth:`~MainAppBase.change_flow`)
-depends on the new flow id and gets determined by the function
+The name of the event method that gets fired when the app want to change the flow (via the method
+:meth:`~MainAppBase.change_flow`) depends on the new flow id and gets determined by the function
 :func:`flow_event_name`.
 
-If the event handler exists and does return `True` for to
-confirm the flow change then flow id and path
-will be changed accordingly.
+If the event handler exists and does return `True` for to confirm the flow change then flow id and path will be changed
+accordingly.
 
-If the flow-specific event call results in a boolean `False` or `None`
-then a more generic event :meth:`~MainAppBase.on_flow_change`
-will be called.
+If the flow-specific event call results in a boolean `False` or `None` then a more generic event
+:meth:`~MainAppBase.on_flow_change` will be called.
 
-If also the generic event returns `False` then the action
-of the new flow id will be searched within
-:data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION` and if not found
-then the flow change will be rejected and :meth:`~MainAppBase.change_flow`
-returns `False`
+If also the generic event returns `False` then the action of the new flow id will be searched within
+:data:`ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION` and if not found then the flow change will be rejected and
+:meth:`~MainAppBase.change_flow` returns `False`
 
-If a confirmed flow change is resulting in a `'focus'` flow action then
-the event `on_flow_widget_focused` will fired and can be used by the
-GUI framework for to set the focus to the widget associated to the
-focus flow id.
+If a confirmed flow change is resulting in a `'focus'` flow action then the event `on_flow_widget_focused` will fired
+and can be used by the GUI framework for to set the focus to the widget associated to the focus flow id.
 
 
 key press events
 ----------------
 
-For to provide key press events to the applications that will use the new
-GUI framework you have to catch the key press events of the framework,
-convert/normalize them and then call the :meth:`~MainAppBase.key_press_from_framework`
-with the normalized modifiers and key args.
+For to provide key press events to the applications that will use the new GUI framework you have to catch the key press
+events of the framework, convert/normalize them and then call the :meth:`~MainAppBase.key_press_from_framework` with the
+normalized modifiers and key args.
 
-The :paramref:`~MainAppBase.key_press_from_framework.modifiers` arg is a string
-that can contain several of the following sub-strings, always in the alphabetic
-order (like listed below):
+The :paramref:`~MainAppBase.key_press_from_framework.modifiers` arg is a string that can contain several of the
+following sub-strings, always in the alphabetic order (like listed below):
 
 * Alt
 * Ctrl
 * Meta
 * Shift
 
-The :paramref:`~MainAppBase.key_press_from_framework.key` arg is a string
-that is specifying the last pressed key. If the key is not representing
-not a single character but a command key, then `key` will be one of the
-following strings:
+The :paramref:`~MainAppBase.key_press_from_framework.key` arg is a string that is specifying the last pressed key. If
+the key is not representing not a single character but a command key, then `key` will be one of the following strings:
 
 * escape
 * tab
@@ -296,59 +228,46 @@ following strings:
 * pgup
 * pgdown
 
-On call of :meth:`~MainAppBase.key_press_from_framework` this method
-will try to dispatch the key press event to your application. First it will
-check the app instance if it has declared a method with the name
+On call of :meth:`~MainAppBase.key_press_from_framework` this method will try to dispatch the key press event to your
+application. First it will check the app instance if it has declared a method with the name
 `on_key_press_of_<modifiers>_<key>` and if so it will call this method.
 
-If this method does return False (or any other value resulting in False)
-then :meth:`~MainAppBase.key_press_from_framework` will check for
-a method with the same name in lower-case and if exits it will call
-this method.
+If this method does return False (or any other value resulting in False) then method
+:meth:`~MainAppBase.key_press_from_framework` will check for a method with the same name in lower-case and if exits
+it will call this method.
 
-If also the second method does return False, then it will try to call the
-event method `on_key_press` of the app instance (if exists) with the modifiers
-and the key as arguments.
+If also the second method does return False, then it will try to call the event method `on_key_press` of the app
+instance (if exists) with the modifiers and the key as arguments.
 
-If the `on_key_press` method does also return False then
-:meth:`~MainAppBase.key_press_from_framework` will finally pass the key
-press event to the original key press handler of the GUI framework
-for further processing.
+If the `on_key_press` method does also return False then :meth:`~MainAppBase.key_press_from_framework` will finally pass
+the key press event to the original key press handler of the GUI framework for further processing.
 
 
 integrate new gui framework
 ---------------------------
 
-For to integrate a new Python GUI framework you have to declare a
-new class that inherits from :class:`MainAppBase` and implements at
-least the abstract method :meth:`~MainAppBase.init_app`.
+For to integrate a new Python GUI framework you have to declare a new class that inherits from :class:`MainAppBase` and
+implements at least the abstract method :meth:`~MainAppBase.init_app`.
 
 A minimal implementation of this method would look like::
 
     def init_app(self):
         return None, None
 
-Most GUI frameworks are providing classes that need to
-be instantiated for the application, the root layout of the app
-and for each window.
-For to keep a reference to these instances within your main app class
-you can use the attributes :attr:`~MainAppBase.framework_app`,
-:attr:`~MainAppBase.framework_root` and
-:attr:`~MainAppBase.framework_win` of :class:`MainAppBase`.
+Most GUI frameworks are providing classes that need to be instantiated for the application, the root layout of the app
+and for each window. For to keep a reference to these instances within your main app class you can use the attributes
+:attr:`~MainAppBase.framework_app`, :attr:`~MainAppBase.framework_root` and :attr:`~MainAppBase.framework_win` of
+:class:`MainAppBase`.
 
-The initialization of the attributes :attr:`~MainAppBase.framework_app`,
-:attr:`~MainAppBase.framework_root` and :attr:`~MainAppBase.framework_win`
-is optional and can be done e.g. within :meth:`~MainAppBase.init_app`
-or in the later called `build` method of the framework app instance.
+The initialization of the attributes :attr:`~MainAppBase.framework_app`, :attr:`~MainAppBase.framework_root` and
+:attr:`~MainAppBase.framework_win` is optional and can be done e.g. within :meth:`~MainAppBase.init_app` or in the later
+called `build` method of the framework app instance.
 
 .. note::
-    If :attr:`~MainAppBase.framework_win` is set to a window instance,
-    then the window instance has to provide
-    a `close` method, which will be called automatically by the
-    :meth:`~MainAppBase.stop_app`.
+    If :attr:`~MainAppBase.framework_win` is set to a window instance, then the window instance has to provide a `close`
+    method, which will be called automatically by the :meth:`~MainAppBase.stop_app`.
 
-A typical implementation of a framework-specific main app class
-looks like::
+A typical implementation of a framework-specific main app class looks like::
 
     from new_gui_framework import NewFrameworkApp, MainWindowClassOfNewFramework
 
@@ -359,29 +278,24 @@ looks like::
 
             return self.framework_app.start, self.framework_app.stop
 
-:meth:`~MainAppBase.init_app` will be executed only once
-at the main app class instantiation. Only the main app instance
-has to initialize the GUI framework for to prepare the app startup
-and has to return at least a callable for to start the event loop
-of the GUI framework.
+:meth:`~MainAppBase.init_app` will be executed only once at the main app class instantiation. Only the main app instance
+has to initialize the GUI framework for to prepare the app startup and has to return at least a callable for to start
+the event loop of the GUI framework.
 
 .. hint::
-    Although not recommended because of possible namespace conflicts,
-    one could e.g. alternatively integrate the framework application class
-    as a mixin to the main app class.
+    Although not recommended because of possible namespace conflicts, one could e.g. alternatively integrate the
+    framework application class as a mixin to the main app class.
 
-For to finally startup the app the :meth:`~MainAppClass.run_app` method
-has to be called from the main module of your app project.
-:meth:`~MainAppBase.run_app` will then start the GUI event loop by
-calling the first method that got returned by :meth:`~MainAppBase.init_app`.
+For to finally startup the app the :meth:`~MainAppClass.run_app` method has to be called from the main module of your
+app project. :meth:`~MainAppBase.run_app` will then start the GUI event loop by calling the first method that got
+returned by :meth:`~MainAppBase.init_app`.
 
 
 optional configuration and extension
 ------------------------------------
 
-Most of the base implementation helper methods can be overwritten by
-either the inheriting framework portion or directly by user main app
-class.
+Most of the base implementation helper methods can be overwritten by either the inheriting framework portion or directly
+by user main app class.
 """
 from abc import ABC, abstractmethod
 import re
@@ -398,7 +312,7 @@ from ae.core import DEBUG_LEVELS                                    # type: igno
 from ae.console import ConsoleApp                                   # type: ignore
 
 
-__version__ = '0.1.39'
+__version__ = '0.1.41'
 
 
 AppStateType = Dict[str, Any]                   #: app state config variable type
@@ -407,7 +321,7 @@ APP_STATE_SECTION_NAME = 'aeAppState'           #: config section name for to st
 
 #: config variable name for to store the current application state version
 APP_STATE_VERSION_VAR_NAME = 'app_state_version'
-APP_STATE_CURRENT_VERSION = 3                   #: current application state version
+APP_STATE_CURRENT_VERSION: int = 3              #: current application state version
 
 MIN_FONT_SIZE = 15.0                            #: minimum font size in pixels
 MAX_FONT_SIZE = 99.0                            #: maximum font size in pixels
@@ -428,12 +342,12 @@ FLOW_KEY_SEP = ':'
 FLOW_ACTION_RE = re.compile("[a-z0-9]+")
 FLOW_OBJECT_RE = re.compile("[A-Za-z0-9_]+")
 
-ACTIONS_EXTENDING_FLOW_PATH = ['add', 'confirm', 'edit', 'enter', 'open', 'show']
+ACTIONS_EXTENDING_FLOW_PATH = ['add', 'confirm', 'edit', 'enter', 'open', 'show', 'suggest']
 ACTIONS_REDUCING_FLOW_PATH = ['close', 'leave']
-ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION = ['', 'enter', 'focus', 'leave']
+ACTIONS_CHANGING_FLOW_WITHOUT_CONFIRMATION = ['', 'enter', 'focus', 'leave', 'suggest']
 
 
-def id_of_flow(action: str, obj: str, key: str = "") -> str:
+def id_of_flow(action: str, obj: str = "", key: str = "") -> str:
     """ create flow id string.
 
     :param action:      flow action string.
@@ -523,15 +437,16 @@ def replace_flow_action(flow_id: str, new_action: str):
     :param new_action:      action to be set/replaced within passed flow id.
     :return:                flow id with new action and object/key from passed flow id.
     """
-    return new_action + NAME_PARTS_SEP + flow_action_split(flow_id)[1]
+    return id_of_flow(new_action, *flow_key_split(flow_action_split(flow_id)[1]))
 
 
 class MainAppBase(ConsoleApp, ABC):
     """ abstract base class for to implement a GUIApp-conform app class """
     # app states
+    app_state_version: int = 0                              #: version number of the app state variables in <config>.ini
     flow_id: str = ""                                       #: id of the current app flow (entered by the app user)
     flow_path: List[str]                                    #: list of flow ids, reflecting recent user actions
-    font_size: float = 30.0                                 #: font size used for toolbar and flow screens
+    font_size: float = 21.0                                 #: font size used for toolbar and flow screens
     lang_code: str = ""                                     #: optional language code (e.g. 'es_ES' for Spanish)
     light_theme: bool = False                               #: True=light theme/background, False=dark theme
     sound_volume: float = 0.12                              #: sound volume of current app (0.0=mute, 1.0=max)
@@ -552,7 +467,7 @@ class MainAppBase(ConsoleApp, ABC):
         :param console_app_kwargs:  kwargs to be passed to the __init__ method of :class:`~ae.console_app.ConsoleApp`.
         """
         self._exit_code: int = 0                            #: init by stop_app() and passed onto OS by run_app()
-        self._last_focus_flow_id: str = id_of_flow('', '')  #: id of the last valid focused window/widget/item/context
+        self._last_focus_flow_id: str = id_of_flow('')      #: id of the last valid focused window/widget/item/context
 
         self._start_event_loop: Optional[Callable]          #: callable to start event loop of GUI framework
         self._stop_event_loop: Optional[Callable]           #: callable to start event loop of GUI framework
@@ -561,8 +476,6 @@ class MainAppBase(ConsoleApp, ABC):
 
         super().__init__(**console_app_kwargs)
 
-        self.load_app_states()
-
         self.call_method('on_app_init')
 
         self.load_images()
@@ -570,6 +483,8 @@ class MainAppBase(ConsoleApp, ABC):
 
         self._start_event_loop, self._stop_event_loop = self.init_app()
 
+        self.load_app_states()
+
     @abstractmethod
     def init_app(self, framework_app_class: Any = None) -> Tuple[Optional[Callable], Optional[Callable]]:
         """ initialize framework app instance and root window/layout, return GUI event loop start/stop methods.
@@ -609,23 +524,45 @@ class MainAppBase(ConsoleApp, ABC):
         :param is_app_state:    pass True for an app state observable.
         """
         setattr(self, name, value)
-        if self.framework_app:
-            if is_app_state:
-                if hasattr(self.framework_app, 'app_states'):       # has observable DictProperty duplicates
-                    # noinspection PyUnresolvedReferences
-                    self.framework_app.app_states[name] = value
-                name = 'app_state_' + name
-            if hasattr(self.framework_app, name):                   # has observable attribute duplicate
-                setattr(self.framework_app, name, value)
+        if is_app_state:
+            if hasattr(self.framework_app, 'app_states'):       # has observable DictProperty duplicates
+                self.framework_app.app_states[name] = value
+            name = 'app_state_' + name
+        if hasattr(self.framework_app, name):                   # has observable attribute duplicate
+            setattr(self.framework_app, name, value)
 
     def load_app_states(self):
-        """ load application state for to prepare app.run_app """
+        """ load application state from the config files for to prepare app.run_app """
         app_state = dict()
         for key in self.app_state_keys():
             app_state[key] = self.get_var(key, section=APP_STATE_SECTION_NAME)
 
         self.setup_app_states(app_state)
 
+    def on_app_init(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_init default/fallback event handler called")
+
+    def on_app_start(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_start default/fallback event handler called")
+
+    def on_app_run(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_run default/fallback event handler called")
+
+    def on_app_stop(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_stop default/fallback event handler called")
+
+    def on_app_exit(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_exit default/fallback event handler called")
+
+    def on_app_quit(self):
+        """ default/fallback event handler. """
+        self.vpo("MainAppBase.on_app_quit default/fallback event handler called")
+
     def on_debug_level_change(self, debug_level: str, _event_kwargs: Dict[str, Any]) -> bool:
         """ language app state change event handler.
 
@@ -676,7 +613,7 @@ class MainAppBase(ConsoleApp, ABC):
         return True
 
     def retrieve_app_states(self) -> AppStateType:
-        """ determine the state of a running app from the config files and return it as dict.
+        """ determine the state of a running app from the main app instance and return it as dict.
 
         :return:    dict with all app states available in the config files.
         """
@@ -705,7 +642,7 @@ class MainAppBase(ConsoleApp, ABC):
                 state = new_state
 
             if key == 'flow_id' and flow_action(state) != 'focus':
-                state = id_of_flow('', '')
+                state = id_of_flow('')
             elif key == 'flow_path':
                 state = [flow_id for flow_id in state if flow_action(flow_id) == 'enter']
 
@@ -763,15 +700,6 @@ class MainAppBase(ConsoleApp, ABC):
         if config_file_version < APP_STATE_CURRENT_VERSION:
             upd_app_state(APP_STATE_VERSION_VAR_NAME, APP_STATE_CURRENT_VERSION)
 
-    def _update_observable_app_states(self, app_state: AppStateType):
-        """ update all the observable app states.
-
-        :param app_state:   dict of app states.
-        """
-        if self.framework_app:
-            for key, val in app_state.items():
-                self.change_observable(key, val, is_app_state=True)
-
     # flow helper methods
 
     @staticmethod
@@ -792,17 +720,20 @@ class MainAppBase(ConsoleApp, ABC):
         :param popups_to_close: optional sequence of widgets to be closed on confirmed flow change.
         :param event_kwargs:    optional args to pass additional data or info onto and from the event handler.
 
-                                info pass onto event handler (e.g. from :attr:`~ae.kivy_app.FlowButton.tap_kwargs`
+                                info pass onto event handler (e.g. via :attr:`~ae.kivy_app.FlowButton.tap_kwargs`
                                 or :attr:`~ae.kivy_app.FlowToggler.tap_kwargs`):
-                                    * `popup_kwargs`: optional dict passed to the Popup `__init__` method,
-                                      like e.g. dict(parent=parent_widget_of_popup, data=...).
-                                    * `tap_widget`: optional tapped button widget instance (initiating this
-                                      flow change).
+
+                                * `popup_kwargs`: optional dict passed to the Popup `__init__` method,
+                                  like e.g. dict(parent=parent_widget_of_popup, data=...).
+                                * `tap_widget`: optional tapped button widget instance (initiating this
+                                  flow change).
+                                * `edit_widget`: optional widget instance for edit/input.
 
                                 info passed from the event handler:
-                                    * `flow_id`: process :attr:`~MainAppBase.flow_path` as specified by the
-                                      :paramref:`~change_flow.new_flow_id` argument, but then overwrite this
-                                      flow id with this event arg value for to set :attr:`~MainAppBase.flow_id`.
+
+                                * `flow_id`: process :attr:`~MainAppBase.flow_path` as specified by the
+                                  :paramref:`~change_flow.new_flow_id` argument, but then overwrite this
+                                  flow id with this event arg value for to set :attr:`~MainAppBase.flow_id`.
 
         :return:                True if flow changed and got confirmed by a declared custom event handler
                                 (either event method or Popup class) of the app, else False.
@@ -824,8 +755,11 @@ class MainAppBase(ConsoleApp, ABC):
             return False
 
         has_flow_focus = flow_action(self.flow_id) == 'focus'
-        empty_flow = id_of_flow('', '')
+        empty_flow = id_of_flow('')
         if action in ACTIONS_EXTENDING_FLOW_PATH:
+            if action == 'edit' and self.flow_path_action() == 'edit' \
+                    and flow_key_split(self.flow_path[-1])[0] == flow_key_split(new_flow_id)[0]:
+                self.flow_path.pop()
             self.flow_path.append(new_flow_id)
             self.change_app_state('flow_path', self.flow_path)
             flow_id = empty_flow if action == 'enter' else new_flow_id
